import numpy as np

def test_kernel_hashes():
    """Compare the hashes fo the kernels against some known working
    version
    """

    print("Not finished yet")
    np.testing.assert_equal(1, 1)

def test_images_hashes():
    """Compare hashes of all the NEAR imagery
    """
    print("Not implemented yet")
    np.testing.assert_equal(1, 1)
    
